/*
    Реализация спецификаций CLDC версии 1.1 (JSR-139), MIDP версии 2.1 (JSR-118)
    и других спецификаций для функционирования компактных приложений на языке
    Java (мидлетов) в среде программного обеспечения Малик Эмулятор.

    Copyright © 2016–2017, 2019–2022 Малик Разработчик

    Это свободная программа: вы можете перераспространять ее и/или изменять
    ее на условиях Меньшей Стандартной общественной лицензии GNU в том виде,
    в каком она была опубликована Фондом свободного программного обеспечения;
    либо версии 3 лицензии, либо (по вашему выбору) любой более поздней версии.

    Эта программа распространяется в надежде, что она будет полезной,
    но БЕЗО ВСЯКИХ ГАРАНТИЙ; даже без неявной гарантии ТОВАРНОГО ВИДА
    или ПРИГОДНОСТИ ДЛЯ ОПРЕДЕЛЕННЫХ ЦЕЛЕЙ. Подробнее см. в Меньшей Стандартной
    общественной лицензии GNU.

    Вы должны были получить копию Меньшей Стандартной общественной лицензии GNU
    вместе с этой программой. Если это не так, см.
    <https://www.gnu.org/licenses/>.
*/

package malik.emulator.microedition.lcdui;

import javax.microedition.lcdui.*;
import malik.emulator.application.*;

public abstract class ScrollBarStyle extends Object
{
    public static int getSliderLength(int page, int range, int trackLength, int minimumSliderLength) {
        /*
         *  Результат этой функции выражается из пропорции:
         *  page      result                   page * trackLength
         *  ----- = -----------  -->  result = ------------------
         *  range   trackLength                      range
         */
        return range == 0 ? 0 : Math.max(minimumSliderLength, (int) ((long) page * (long) trackLength / (long) range));
    }

    public static int getSliderPosition(int page, int range, int position, int trackLength, int sliderLength) {
        /*
         *  Результат этой функции выражается из пропорции:
         *    position               result
         *  ------------ = --------------------------
         *  range - page   trackLength - sliderLength
         *  |
         *  v
         *           position * (trackLength - sliderLength)
         *  result = ---------------------------------------
         *                        range - page
         */
        return range == page ? 0 : (int) ((long) position * (long) (trackLength - sliderLength) / (long) (range - page));
    }

    private final int size;

    protected ScrollBarStyle(int size) {
        this.size = size < 1 ? 1 : size;
    }

    public abstract void horizontalScrollBarPointerEvent(ScrollBar scrollbar, PointerEvent event, int width, int height);

    public abstract void horizontalScrollBarPaintEvent(ScrollBar scrollbar, Graphics render, int width, int height);

    public abstract void verticalScrollBarPointerEvent(ScrollBar scrollbar, PointerEvent event, int width, int height);

    public abstract void verticalScrollBarPaintEvent(ScrollBar scrollbar, Graphics render, int width, int height);

    public void sizeGripPaintEvent(boolean sizeChangeAllowed, Graphics render, int width, int height) {
    }

    public final int size() {
        return size;
    }
}
